/*******************************************************************
 * lipo_functions.c
 * Functions for Lipower battery backpack board
 *
 * Written by Chris Taylor 5/20/08
 * Copyright Sparkfun Electronics, 2008
 ******************************************************************/

#include <msp430x12x2.h>
#include <msp430/usart.h>
#include <msp430/timera.h>
#include <msp430/adc10.h>
#include "lipo_functions.h"


/* Voltage vars */
volatile int current_voltage;
volatile int volt_data1[16];
volatile int volt_data2[16];
volatile int volt_ref1 = 0;
volatile int volt_ref2 = 0;


/* Array of process request functions */
int (*request_functions[])(void) =
{
    get_gen_info, 	//0
    get_power_status,	//1
    out1_on,		//2
    out1_off,		//3
    out2_on,		//4
    out2_off,		//5
    out3_on, 		//6
    out3_off,		//7
    reset_MSP		//8
};


/******************************************************************/
int wastetime(int time)
{
    int i = 0;
    int j = 0;
    while(i < time)
    {
        for(j=0;j<1000;j++)
            asm("nop");
        i++;
    }
}
/******************************************************************/


/******************************************************************/
int sleep(void)
{
    _BIS_SR(GIE);
    _BIS_SR(CPUOFF);
}
/******************************************************************/


/******************************************************************/
int check_char(char incoming)
{
    if((incoming >= 0) && (incoming <= 8))
    {
        return request_functions[incoming]();
    }
    else
    {
        return '?';
    }
}
/******************************************************************/


/******************************************************************/

int store_voltage(int volt)
{
    volt_data1[volt_ref1] = volt;
    volt_ref1++;
    volt_ref1 &= 0x000f;

    if(volt_ref1 == 0x000f);
    {
        volt_data2[volt_ref2] = volt_avg(volt_data1);
        volt_ref2++;
        volt_ref2 &= 0x000f;
    }
}


int volt_avg(volatile int *array)
{
    int i = 0;
    unsigned int total = 0;
    for(i = 0;i<16;i++)
    {
        total += array[i];
    }
    total = total/16;

    return total;
}
/******************************************************************/


int gen_info(void)
{
    /* >>>>> get battery voltage <<<<<<< */
    int v = volt_avg(volt_data2);
    int mask = 0x00;
    int in0, in1, in2, in3;
    if(v >= 0x02b2)               /* 3 bars */
        mask |= 0x60;
    else if((v > 0x029c) && (v <= 0x02b2)) /* 2 bars */
        mask |= 0x40;
    else if((v > 0x027f) && (v <= 0x029c)) /* 1 bar */
        mask |= 0x20;


    /* >>>>> get charge status <<<<<<< */
    if((P2IN & 0x0002) != 0x0002)
        mask |= 0x10;


    return mask;
}

/*****************************************************************/

int power_status(void)
{
    int mask = 0x0000;
    if((P2OUT & 0x0008) == 0x0008)
        mask |= 0x0004;
    if((P2OUT & 0x0010) == 0x0010)
        mask |= 0x0001;
    if((P3OUT & 0x0040) == 0x0040)
        mask |= 0x0002;
    return mask;
}
/*****************************************************************/


/*****************************************************************/
int power_up_down(int module, int updown)
{
    if(updown)
    {
        switch(module)
        {
            case 0:
                P2OUT |= 0x0010; 
                break;
            case 1:
                P2OUT |= 0x0008; 
                break;
            case 2:
                P3OUT |= 0x0040; 
                break;
        }
    }
    else
    {
        switch(module)
        {
            case 0:
                P2OUT &= ~(0x0010);
                break;
            case 1:
                P2OUT &= ~(0x0008);
                break;
            case 2:
                P3OUT &= ~(0x0040);
                break;
            case 3:
                WDTCTL = 0x0000;
                break;
        }
    }
}
/******************************************************************/


int get_gen_info(void) { return gen_info(); }
int get_power_status(void) { return power_status(); }

int out1_on(void) { power_up_down(0,1); return 2; }
int out1_off(void){ power_up_down(0,0); return 3; }

int out2_on(void) { power_up_down(2,1); return 4; }
int out2_off(void) { power_up_down(2,0); return 5; }

int out3_on(void) { power_up_down(1,1); return 6; }
int out3_off(void) { power_up_down(1,0); return 7; }

int reset_MSP(void) { power_up_down(3,0); return 8; }

int check_charge(void)
{
    if((P2IN & 0x0002) != 0x0002)
        P1OUT |= 0x0010;
    else
        P1OUT &= ~(0x0010);
}


